
#include <Standard_StdAllocator.hxx>

class Standard_StdAllocatorTest: public QObject
{
    Q_OBJECT

private slots:

    void Traits();
    void Containers();
};



void Standard_StdAllocatorTest::Traits()
{
    //type definitions
    typedef Handle_Standard_Transient elem_type;
    typedef Standard_StdAllocator<elem_type> allocator_type;

    QCOMPARE (sizeof (allocator_type::value_type),sizeof (elem_type));
    QCOMPARE (sizeof (allocator_type::pointer),sizeof (void*));
    QCOMPARE (sizeof (allocator_type::const_pointer),sizeof (void*));

    elem_type aDummy;
    allocator_type::reference aRef = aDummy;
    allocator_type::const_reference aConstRef = aDummy;

    QCOMPARE (sizeof (allocator_type::size_type),sizeof (size_t));
    QCOMPARE (sizeof (allocator_type::difference_type),sizeof (ptrdiff_t));

    typedef int other_elem_type;
    QCOMPARE (sizeof (allocator_type::rebind<other_elem_type>::other::value_type), sizeof (other_elem_type));
}

void Standard_StdAllocatorTest::Containers()
{
    //typed allocator
    std::list<TopoDS_Shape, Standard_StdAllocator<TopoDS_Shape> > aL;
    TopoDS_Solid aSolid = BRepPrimAPI_MakeBox (10., 20., 30.);
    aL.push_back (aSolid);
    QCOMPARE (aL.size(), size_t (1));

    //type cast
    Standard_StdAllocator<char> aCAlloc;
    std::vector<int, Standard_StdAllocator<int> > aV (aCAlloc);
    aV.push_back (1);
    QCOMPARE (aV.size(), size_t (1));

    //using void-specialization allocator
    std::vector<int, Standard_StdAllocator<void> > aV2;
    aV2.resize (10);
    aV2.push_back (-1);
    QCOMPARE (aV2.size(), size_t (11));
}
